@@ -2,8 +2,8 @@
 # sgml-catalog-check.pl -- check sgml catalog file
 #
 #Author: apharris@onshore.com (A. P. Harris)
-#$Date: 1999/07/14 08:03:45 $
-#$Revision: 1.11 $
+#$Date: 2001/11/20 03:49:32 $
+#$Revision: 1.18 $
 #
 #Todo:
 #	cross check the links/dtds and make sure they all appear in the
@@ -15,9 +15,10 @@
 use Getopt::Std;
 
 $Verbose = 1;			# verboseness, 1 == chatty, 2 == loud
-$SGMLdir = "debian/tmp/usr/lib/sgml"; # default dir for link making etc
+$SGMLdir = "debian/tmp/usr/share/sgml"; # default dir for link making etc
 $Catalog = "sgml.catalog";	# default SGML catalog file
 $MakeLinks = 0;			# whether to make the SGML links
+$ChopEN = 1;			# whether to chop off //EN[//.*] language specifiers
 @MakeDir = ("install", "-d", "-o", "root", "-g", "root", 
 	    "-m", "755");	# how to make a directory recursively
 
@@ -28,12 +29,13 @@ file actually exists.
    -l                   create SGML links if they do not exist
    -d <SGML dir>        base dir, default is $SGMLdir
    -v <number>		verbosity amount, 0=silent, 1=default, 2=debug
+   -e                   don't omit the trailing EN language specifier (//EN)
    <SGML catalog file>  default is $Catalog
 ";
 
 $warnings = $errors = 0;	# error and warning count
 
-&getopts('hlv:d:') || die $Usage;
+&getopts('ehlv:d:') || die $Usage;
 
 if ( $opt_h ) 
 {
@@ -55,11 +57,14 @@ if ( $opt_l ) {
     $MakeLinks = $opt_l;
 }
 
-if ( $#ARGV > 0 )
-{
-    die "too many arguments\n$Usage";
+if ( $opt_e ) {
+    $opt_e = $opt_e;            # shut up, -w
+    $ChopEN = 0;
 }
-elsif ( $#ARGV == 0 ) {
+
+if ( $#ARGV > 0 ) {
+    die "too many arguments\n$Usage";
+} elsif ( $#ARGV == 0 ) {
     $Catalog = $ARGV[0];
 }
 
@@ -71,14 +76,24 @@ elsif ( $#ARGV == 0 ) {
 open(CAT, "<$Catalog") or
     die "cannot read $Catalog: $!\n";
 
+## when checking for system ids, we need to check relative to the
+## catalog file location, so figure out the relative dir of the
+## catalog file, possibly removing a prepended SGMLdir
+
+$CatDir = `dirname $Catalog`;
+chomp($CatDir);
+$CatDir =~ s/^$SGMLdir\/?//;
+
 while (<CAT>) {
     chomp;
-    if ( m/^PUBLIC\s+\"([^\"]+)\"\s+\"?([^\s"]+)\"?/ ) {
+    if ( m/^PUBLIC\s+\"([^\"]+)\"\s+\"?([^\s\"]+)\"?/ ) {
 	( $id, $file ) = ( $1, $2 );
-	debug("found identifier \"$id\"");
-	debug("source file is $file");
-	if ( ! -f "$SGMLdir/$file" ) {
-	    error("referenced-file-does-not-exist $SGMLdir/$file");
+	debug("found public identifier \"$id\"");
+	debug("system identifier is $file");
+	if ( -f "$SGMLdir/$CatDir/$file" ) {
+            $file = "$CatDir/$file";
+        } elsif ( ! -f "$SGMLdir/$file" ) {
+	    error("referenced-file-does-not-exist $SGMLdir/$CatDir/$file of $SGMLdir/$file");
 	    next;
 	}
 	
@@ -86,12 +101,10 @@ while (<CAT>) {
 	    ( $reg, $vendor, $type, $name, $misc ) = ( $1, $2, $3, $4, $5 );
 
 	    if ( $type eq "ENTITIES" ) {
-		( ! $file =~ m!^entities/! ) &&
-		    error("SGML-entity-not-in-entities-dir $file");
+                                # AOK, no checking for location
 	    } 
 	    elsif ( $type eq "DTD" || $type eq "ELEMENTS" ) {
-		( ! $file =~ m!^dtd/! ) &&
-		    error("SGML-DTD-not-in-dtd-dir $file");
+                                # AOK, no checking for location
 	    }
 	    elsif ( $type eq "DOCUMENT" ) {
 		( $file =~ m!^dtd/! || $file =~ m!^entities! ) &&
@@ -109,13 +122,15 @@ while (<CAT>) {
 	    $link =~ s!^(\-|\+)//!!;	# registered mark doesnt need dir
 	    $link =~ s/  / /g;		# compact spaces
 	    $link =~ s![ %]!_!g;	# convert spaces/% to _
-	    $link =~ s!//EN$!!;		# remove language specifier
-	    $link =~ s!//EN//.*$!!;	# remove language specifier plus
+            if ( $ChopEN ) {
+                $link =~ s!//EN$!!;     # remove language specifier
+                $link =~ s!//EN//.*$!!;	# remove language specifier plus
+            }
 	    $link =~ s!//!\0!g;		# convert // to NULL
 	    $link =~ s!/!_!g;		# convert slash to _
 	    $link =~ s!\0!/!g;		# convert NULL to /
 	    $link =~ s!/DTD_!/dtd/!;	# it is a DTD
-	    $link =~ s!/ELEMENTS_!/dtd/!; # it is a DTD
+	    $link =~ s!/ELEMENTS_!/elements/!; # it is an element
 	    $link =~ s!/ENTITIES_!/entities/!; # it is an entity
 	    debug("checking link $link");
 	    if ( -l "$SGMLdir/$link" ) {
@@ -130,7 +145,7 @@ while (<CAT>) {
 		if ( ! -d $dest ) {
 		    inform("creating dir $dest");
 		    system((@MakeDir, $dest)) == 0 or
-			die("cannot create dir $dest: $?\n  command was " . join(" ", @MakeDir) . "\n");
+			die("cannot create dir '$dest': $?\n  command was " . join(" ", @MakeDir) . " $dest\n");
 		}
 		# now lets formulate what to link to
 		($dest = $link) =~ s![^/]!!g; # remove non /
@@ -186,3 +201,4 @@ sub error {
     $errors++;
     warn("E: $msg\n");
 }
+
